@@ -18,7 +18,6 @@
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
-
 import xml.sax
 import datetime
 import itertools
@@ -46,7 +45,8 @@ class MTurkConnection(AWSQueryConnection):
                  is_secure=True, port=None, proxy=None, proxy_port=None,
                  proxy_user=None, proxy_pass=None,
                  host=None, debug=0,
-                 https_connection_factory=None):
+                 https_connection_factory=None, security_token=None,
+                 profile_name=None):
         if not host:
             if config.has_option('MTurk', 'sandbox') and config.get('MTurk', 'sandbox') == 'True':
                 host = 'mechanicalturk.sandbox.amazonaws.com'
@@ -54,11 +54,13 @@ class MTurkConnection(AWSQueryConnection):
                 host = 'mechanicalturk.amazonaws.com'
         self.debug = debug
 
-        AWSQueryConnection.__init__(self, aws_access_key_id,
+        super(MTurkConnection, self).__init__(aws_access_key_id,
                                     aws_secret_access_key,
                                     is_secure, port, proxy, proxy_port,
                                     proxy_user, proxy_pass, host, debug,
-                                    https_connection_factory)
+                                    https_connection_factory,
+                                    security_token=security_token,
+                                    profile_name=profile_name)
 
     def _required_auth_capability(self):
         return ['mturk']
@@ -304,7 +306,7 @@ class MTurkConnection(AWSQueryConnection):
         records, return the page numbers to be retrieved.
         """
         pages = total_records / page_size + bool(total_records % page_size)
-        return range(1, pages + 1)
+        return list(range(1, pages + 1))
 
     def get_all_hits(self):
         """
@@ -385,15 +387,15 @@ class MTurkConnection(AWSQueryConnection):
                 The number of assignments on the page in the filtered results
                 list, equivalent to the number of assignments being returned
                 by this call.
-                A non-negative integer
+                A non-negative integer, as a string.
         PageNumber
                 The number of the page in the filtered results list being
                 returned.
-                A positive integer
+                A positive integer, as a string.
         TotalNumResults
                 The total number of HITs in the filtered results list based
                 on this call.
-                A non-negative integer
+                A non-negative integer, as a string.
 
         The ResultSet will contain zero or more Assignment objects
 
@@ -438,6 +440,21 @@ class MTurkConnection(AWSQueryConnection):
             params['RequesterFeedback'] = feedback
         return self._process_request('ApproveRejectedAssignment', params)
 
+    def get_file_upload_url(self, assignment_id, question_identifier):
+        """
+        Generates and returns a temporary URL to an uploaded file. The
+        temporary URL is used to retrieve the file as an answer to a
+        FileUploadAnswer question, it is valid for 60 seconds.
+
+        Will have a FileUploadURL attribute as per the API Reference.
+        """
+
+        params = {'AssignmentId': assignment_id,
+                  'QuestionIdentifier': question_identifier}
+
+        return self._process_request('GetFileUploadURL', params,
+                                     [('FileUploadURL', FileUploadURL)])
+
     def get_hit(self, hit_id, response_groups=None):
         """
         """
@@ -826,7 +843,7 @@ class MTurkConnection(AWSQueryConnection):
         """
         body = response.read()
         if self.debug == 2:
-            print body
+            print(body)
         if '<Errors>' not in body:
             rs = ResultSet(marker_elems)
             h = handler.XmlHandler(rs, self)
@@ -875,7 +892,7 @@ class MTurkConnection(AWSQueryConnection):
         return duration
 
 
-class BaseAutoResultElement:
+class BaseAutoResultElement(object):
     """
     Base class to automatically add attributes when parsing XML
     """
@@ -913,6 +930,14 @@ class HIT(BaseAutoResultElement):
     expired = property(_has_expired)
 
 
+class FileUploadURL(BaseAutoResultElement):
+    """
+    Class to extract an FileUploadURL structure from a response
+    """
+
+    pass
+
+
 class HITTypeId(BaseAutoResultElement):
     """
     Class to extract an HITTypeId structure from a response
@@ -955,7 +980,7 @@ class QualificationRequest(BaseAutoResultElement):
     """
 
     def __init__(self, connection):
-        BaseAutoResultElement.__init__(self, connection)
+        super(QualificationRequest, self).__init__(connection)
         self.answers = []
 
     def endElement(self, name, value, connection):
@@ -967,7 +992,7 @@ class QualificationRequest(BaseAutoResultElement):
             xml.sax.parseString(value, h)
             self.answers.append(answer_rs)
         else:
-            BaseAutoResultElement.endElement(self, name, value, connection)
+            super(QualificationRequest, self).endElement(name, value, connection)
 
 
 class Assignment(BaseAutoResultElement):
@@ -980,7 +1005,7 @@ class Assignment(BaseAutoResultElement):
     """
 
     def __init__(self, connection):
-        BaseAutoResultElement.__init__(self, connection)
+        super(Assignment, self).__init__(connection)
         self.answers = []
 
     def endElement(self, name, value, connection):
@@ -992,7 +1017,7 @@ class Assignment(BaseAutoResultElement):
             xml.sax.parseString(value, h)
             self.answers.append(answer_rs)
         else:
-            BaseAutoResultElement.endElement(self, name, value, connection)
+            super(Assignment, self).endElement(name, value, connection)
 
 
 class QuestionFormAnswer(BaseAutoResultElement):
@@ -1016,7 +1041,7 @@ class QuestionFormAnswer(BaseAutoResultElement):
     """
 
     def __init__(self, connection):
-        BaseAutoResultElement.__init__(self, connection)
+        super(QuestionFormAnswer, self).__init__(connection)
         self.fields = []
         self.qid = None
 
