@@ -14,12 +14,15 @@
 # THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 # OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABIL-
 # ITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT
-# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
+# SHALL THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 # WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 # OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
 # IN THE SOFTWARE.
 
-def bucket_lister(bucket, prefix='', delimiter='', marker='', headers=None):
+from boto.compat import urllib, six
+
+def bucket_lister(bucket, prefix='', delimiter='', marker='', headers=None,
+                  encoding_type=None):
     """
     A generator function for listing keys in a bucket.
     """
@@ -27,14 +30,19 @@ def bucket_lister(bucket, prefix='', delimiter='', marker='', headers=None):
     k = None
     while more_results:
         rs = bucket.get_all_keys(prefix=prefix, marker=marker,
-                                 delimiter=delimiter, headers=headers)
+                                 delimiter=delimiter, headers=headers,
+                                 encoding_type=encoding_type)
         for k in rs:
             yield k
         if k:
             marker = rs.next_marker or k.name
+        if marker and encoding_type == "url":
+            if isinstance(marker, six.text_type):
+                marker = marker.encode('utf-8')
+            marker = urllib.parse.unquote(marker)
         more_results= rs.is_truncated
-        
-class BucketListResultSet:
+
+class BucketListResultSet(object):
     """
     A resultset for listing keys within a bucket.  Uses the bucket_lister
     generator function and implements the iterator interface.  This
@@ -43,20 +51,24 @@ class BucketListResultSet:
     keys in a reasonably efficient manner.
     """
 
-    def __init__(self, bucket=None, prefix='', delimiter='', marker='', headers=None):
+    def __init__(self, bucket=None, prefix='', delimiter='', marker='',
+                 headers=None, encoding_type=None):
         self.bucket = bucket
         self.prefix = prefix
         self.delimiter = delimiter
         self.marker = marker
         self.headers = headers
+        self.encoding_type = encoding_type
 
     def __iter__(self):
         return bucket_lister(self.bucket, prefix=self.prefix,
                              delimiter=self.delimiter, marker=self.marker,
-                             headers=self.headers)
+                             headers=self.headers,
+                             encoding_type=self.encoding_type)
 
 def versioned_bucket_lister(bucket, prefix='', delimiter='',
-                            key_marker='', version_id_marker='', headers=None):
+                            key_marker='', version_id_marker='', headers=None,
+                            encoding_type=None):
     """
     A generator function for listing versions in a bucket.
     """
@@ -66,14 +78,14 @@ def versioned_bucket_lister(bucket, prefix='', delimiter='',
         rs = bucket.get_all_versions(prefix=prefix, key_marker=key_marker,
                                      version_id_marker=version_id_marker,
                                      delimiter=delimiter, headers=headers,
-                                     max_keys=999)
+                                     max_keys=999, encoding_type=encoding_type)
         for k in rs:
             yield k
         key_marker = rs.next_key_marker
         version_id_marker = rs.next_version_id_marker
         more_results= rs.is_truncated
-        
-class VersionedBucketListResultSet:
+
+class VersionedBucketListResultSet(object):
     """
     A resultset for listing versions within a bucket.  Uses the bucket_lister
     generator function and implements the iterator interface.  This
@@ -83,24 +95,26 @@ class VersionedBucketListResultSet:
     """
 
     def __init__(self, bucket=None, prefix='', delimiter='', key_marker='',
-                 version_id_marker='', headers=None):
+                 version_id_marker='', headers=None, encoding_type=None):
         self.bucket = bucket
         self.prefix = prefix
         self.delimiter = delimiter
         self.key_marker = key_marker
         self.version_id_marker = version_id_marker
         self.headers = headers
+        self.encoding_type = encoding_type
 
     def __iter__(self):
         return versioned_bucket_lister(self.bucket, prefix=self.prefix,
                                        delimiter=self.delimiter,
                                        key_marker=self.key_marker,
                                        version_id_marker=self.version_id_marker,
-                                       headers=self.headers)
+                                       headers=self.headers,
+                                       encoding_type=self.encoding_type)
 
 def multipart_upload_lister(bucket, key_marker='',
                             upload_id_marker='',
-                            headers=None):
+                            headers=None, encoding_type=None):
     """
     A generator function for listing multipart uploads in a bucket.
     """
@@ -109,14 +123,15 @@ def multipart_upload_lister(bucket, key_marker='',
     while more_results:
         rs = bucket.get_all_multipart_uploads(key_marker=key_marker,
                                               upload_id_marker=upload_id_marker,
-                                              headers=headers)
+                                              headers=headers,
+                                              encoding_type=encoding_type)
         for k in rs:
             yield k
         key_marker = rs.next_key_marker
         upload_id_marker = rs.next_upload_id_marker
         more_results= rs.is_truncated
-        
-class MultiPartUploadListResultSet:
+
+class MultiPartUploadListResultSet(object):
     """
     A resultset for listing multipart uploads within a bucket.
     Uses the multipart_upload_lister generator function and
@@ -126,14 +141,16 @@ class MultiPartUploadListResultSet:
     keys in a reasonably efficient manner.
     """
     def __init__(self, bucket=None, key_marker='',
-                 upload_id_marker='', headers=None):
+                 upload_id_marker='', headers=None, encoding_type=None):
         self.bucket = bucket
         self.key_marker = key_marker
         self.upload_id_marker = upload_id_marker
         self.headers = headers
+        self.encoding_type = encoding_type
 
     def __iter__(self):
         return multipart_upload_lister(self.bucket,
                                        key_marker=self.key_marker,
                                        upload_id_marker=self.upload_id_marker,
-                                       headers=self.headers)
+                                       headers=self.headers,
+                                       encoding_type=self.encoding_type)
