@@ -27,8 +27,9 @@ Some unit tests for S3 Key
 
 from tests.unit import unittest
 import time
-from boto.compat import six, StringIO, urllib
 
+import boto.s3
+from boto.compat import six, StringIO, urllib
 from boto.s3.connection import S3Connection
 from boto.s3.key import Key
 from boto.exception import S3ResponseError
@@ -459,3 +460,75 @@ class S3KeyTest(unittest.TestCase):
         kn = self.bucket.new_key("testkey_for_sse_c")
         ks = kn.get_contents_as_string(headers=header)
         self.assertEqual(ks, content.encode('utf-8'))
+
+
+class S3KeySigV4Test(unittest.TestCase):
+    def setUp(self):
+        self.conn = boto.s3.connect_to_region('eu-central-1')
+        self.bucket_name = 'boto-sigv4-key-%d' % int(time.time())
+        self.bucket = self.conn.create_bucket(self.bucket_name,
+                                              location='eu-central-1')
+
+    def tearDown(self):
+        for key in self.bucket:
+            key.delete()
+        self.bucket.delete()
+
+    def test_put_get_with_non_string_headers_key(self):
+        k = Key(self.bucket)
+        k.key = 'foobar'
+        body = 'This is a test of S3'
+        # A content-length header will be added to this request since it
+        # has a body.
+        k.set_contents_from_string(body)
+        # Set a header that has an integer. This checks for a bug where
+        # the sigv4 signer assumes that all of the headers are strings.
+        headers = {'Content-Length': 0}
+        from_s3_key = self.bucket.get_key('foobar', headers=headers)
+        self.assertEqual(from_s3_key.get_contents_as_string().decode('utf-8'),
+                         body)
+
+
+class S3KeyVersionCopyTest(unittest.TestCase):
+    def setUp(self):
+        self.conn = S3Connection()
+        self.bucket_name = 'boto-key-version-copy-%d' % int(time.time())
+        self.bucket = self.conn.create_bucket(self.bucket_name)
+        self.bucket.configure_versioning(True)
+        
+    def tearDown(self):
+        for key in self.bucket.list_versions():
+            key.delete()
+        self.bucket.delete()
+        
+    def test_key_overwrite_and_copy(self):
+        first_content = "abcdefghijklm"
+        second_content = "nopqrstuvwxyz"
+        k = Key(self.bucket, 'testkey')
+        k.set_contents_from_string(first_content)
+        # Wait for S3's eventual consistency (may not be necessary)
+        while self.bucket.get_key('testkey') is None:
+            time.sleep(5)
+        # Get the first version_id
+        first_key = self.bucket.get_key('testkey')
+        first_version_id = first_key.version_id
+        # Overwrite the key
+        k = Key(self.bucket, 'testkey')
+        k.set_contents_from_string(second_content)
+        # Wait for eventual consistency
+        while True:
+            second_key = self.bucket.get_key('testkey')
+            if second_key is None or second_key.version_id == first_version_id:
+                time.sleep(5)
+            else:
+                break
+        # Copy first key (no longer the current version) to a new key
+        source_key = self.bucket.get_key('testkey',
+                                          version_id=first_version_id)
+        source_key.copy(self.bucket, 'copiedkey')
+        while self.bucket.get_key('copiedkey') is None:
+            time.sleep(5)
+        copied_key = self.bucket.get_key('copiedkey')
+        copied_key_contents = copied_key.get_contents_as_string()
+        self.assertEqual(first_content, copied_key_contents)
+
