@@ -483,6 +483,19 @@ e3b0c44298fc1c149afbf4c8996fb92427ae41e4649b934ca495991b7852b855"""
         authed_req = self.auth.canonical_request(request)
         self.assertEqual(authed_req, expected)
 
+    def test_non_string_headers(self):
+        self.awesome_bucket_request.headers['Content-Length'] = 8
+        canonical_headers = self.auth.canonical_headers(
+            self.awesome_bucket_request.headers)
+        self.assertEqual(
+            canonical_headers,
+            'content-length:8\n'
+            'user-agent:Boto\n'
+            'x-amz-content-sha256:e3b0c44298fc1c149afbf4c8996fb92427ae'
+            '41e4649b934ca495991b7852b855\n'
+            'x-amz-date:20130605T193245Z'
+        )
+
 
 class FakeS3Connection(object):
     def __init__(self, *args, **kwargs):
@@ -518,8 +531,10 @@ class TestS3SigV4OptIn(MockServiceWithConfigTestCase):
 
     def test_sigv4_non_optional(self):
         # Requires SigV4.
-        fake = FakeS3Connection(host='s3.cn-north-1.amazonaws.com.cn')
-        self.assertEqual(fake._required_auth_capability(), ['hmac-v4-s3'])
+        for region in ['.cn-north', '.eu-central', '-eu-central']:
+            fake = FakeS3Connection(host='s3' + region + '-1.amazonaws.com')
+            self.assertEqual(
+                fake._required_auth_capability(), ['hmac-v4-s3'])
 
     def test_sigv4_opt_in_config(self):
         # Opt-in via the config.
