@@ -134,11 +134,14 @@ class TestCloudFormationUpdateStack(CloudFormationConnectionBase):
         api_response = self.service_connection.update_stack(
             'stack_name', template_url='http://url',
             template_body=SAMPLE_TEMPLATE,
-            parameters=[('KeyName', 'myKeyName')],
+            parameters=[('KeyName', 'myKeyName'), ('KeyName2', "", True),
+                        ('KeyName3', "", False), ('KeyName4', None, True),
+                        ('KeyName5', "Ignore Me", True)],
             tags={'TagKey': 'TagValue'},
             notification_arns=['arn:notify1', 'arn:notify2'],
             disable_rollback=True,
-            timeout_in_minutes=20
+            timeout_in_minutes=20,
+            use_previous_template=True
         )
         self.assert_request_parameters({
             'Action': 'UpdateStack',
@@ -148,6 +151,14 @@ class TestCloudFormationUpdateStack(CloudFormationConnectionBase):
             'NotificationARNs.member.2': 'arn:notify2',
             'Parameters.member.1.ParameterKey': 'KeyName',
             'Parameters.member.1.ParameterValue': 'myKeyName',
+            'Parameters.member.2.ParameterKey': 'KeyName2',
+            'Parameters.member.2.UsePreviousValue': 'true',
+            'Parameters.member.3.ParameterKey': 'KeyName3',
+            'Parameters.member.3.ParameterValue': '',
+            'Parameters.member.4.UsePreviousValue': 'true',
+            'Parameters.member.4.ParameterKey': 'KeyName4',
+            'Parameters.member.5.UsePreviousValue': 'true',
+            'Parameters.member.5.ParameterKey': 'KeyName5',
             'Tags.member.1.Key': 'TagKey',
             'Tags.member.1.Value': 'TagValue',
             'StackName': 'stack_name',
@@ -155,6 +166,7 @@ class TestCloudFormationUpdateStack(CloudFormationConnectionBase):
             'TimeoutInMinutes': 20,
             'TemplateBody': SAMPLE_TEMPLATE,
             'TemplateURL': 'http://url',
+            'UsePreviousTemplate': 'true',
         })
 
     def test_update_stack_with_minimum_args(self):
@@ -698,7 +710,7 @@ class TestCloudFormationSetStackPolicy(CloudFormationConnectionBase):
         self.set_http_response(status_code=200)
         api_response = self.service_connection.set_stack_policy('stack-id',
             stack_policy_body='{}')
-        self.assertEqual(api_response['Some'], 'content')
+        self.assertDictEqual(api_response, {'SetStackPolicyResult': {'Some': 'content'}})
         self.assert_request_parameters({
             'Action': 'SetStackPolicy',
             'ContentType': 'JSON',
