@@ -21,10 +21,9 @@
 # IN THE SOFTWARE.
 #
 import tempfile
-from Queue import Queue
+from boto.compat import Queue
 
-import mock
-from tests.unit import unittest
+from tests.compat import mock, unittest
 from tests.unit import AWSMockServiceTestCase
 
 from boto.glacier.concurrent import ConcurrentUploader, ConcurrentDownloader
@@ -40,8 +39,8 @@ class FakeThreadedConcurrentUploader(ConcurrentUploader):
         self.upload_id = upload_id
 
     def _wait_for_upload_threads(self, hash_chunks, result_queue, total_parts):
-        for i in xrange(total_parts):
-            hash_chunks[i] = 'foo'
+        for i in range(total_parts):
+            hash_chunks[i] = b'foo'
 
 
 class FakeThreadedConcurrentDownloader(ConcurrentDownloader):
@@ -88,7 +87,7 @@ class TestConcurrentUploader(unittest.TestCase):
                                                   'vault_name')
         uploader.upload('foofile')
         q = uploader.worker_queue
-        items = [q.get() for i in xrange(q.qsize())]
+        items = [q.get() for i in range(q.qsize())]
         self.assertEqual(items[0], (0, 4 * 1024 * 1024))
         self.assertEqual(items[1], (1, 4 * 1024 * 1024))
         # 2 for the parts, 10 for the end sentinels (10 threads).
@@ -111,7 +110,7 @@ class TestConcurrentUploader(unittest.TestCase):
         downloader = FakeThreadedConcurrentDownloader(job)
         downloader.download('foofile')
         q = downloader.worker_queue
-        items = [q.get() for i in xrange(q.qsize())]
+        items = [q.get() for i in range(q.qsize())]
         self.assertEqual(items[0], (0, 4 * 1024 * 1024))
         self.assertEqual(items[1], (1, 4 * 1024 * 1024))
         # 2 for the parts, 10 for the end sentinels (10 threads).
